@@ -9,8 +9,6 @@
 #    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
-todo <- function() help( "todo.spam")
-spam.history <- function() help("spam.history")
 
 # We check that
 # 1.  #colindices equals #entries
@@ -68,6 +66,8 @@ validate_spam <- function(object) {
 }
 
 
+
+# Start with the two classes. Remaining part contains `spam` only material.
 setClass(
   Class = "spam",
   slots = c(entries="numeric",
@@ -83,6 +83,24 @@ setClass(
   validity = validate_spam
 )
 
+setClass("spam.chol.NgPeyton",
+         representation(entries="numeric",      colindices="integer",
+                        colpointers="integer",  rowpointers="integer",
+                        dimension="integer",
+                        pivot="integer",        invpivot="integer",
+                        supernodes="integer",   snmember="integer",
+                        memory="integer",       nnzA="integer")
+         )
+
+# lindx=  colindices
+# xlindx= colpointers
+# xlnz=   rowpointers
+# snode=snmember
+# xsuper=supernodes
+# c(... nnztmp,cachesize)= memory
+########################################################################
+
+
 
 # Internal function to create new spam or spam64 object, depending on the size.
 .newSpam <- function(entries=0,
@@ -121,18 +139,119 @@ setClass(
 }
 
 
-print.spam <- function(x,...) {
+# Create a string containing the matrix size with appropriate unit. 
+# not planned to be exported.
+# If length(x)==1, this represents the number of doubles (or other size).
+printSize <- function(x, size=8, digits=2L) {
+    mat <- if (length(x)==1) 8*as.numeric(x) else  as.numeric( object.size(x))
+    unitpower <- max(min( floor( log(mat, 1024)),3),1)
+    paste(round(mat/1024^unitpower, digits=digits), c("Kb","Mb","Gb")[unitpower])
+}
+
+
+.print.spam <- function(x, digits = 2, rowpointer = FALSE, minimal = TRUE, # show row pointers in left margin
+              zerosymbol = "'", ...){
+
+    if (minimal) {
+      print(x@entries, ...)
+      return(invisible(NULL))
+    }
+
+    # never true if used via print.spam
+    if(is.matrix(x)) {
+      x <- as.spam(x) }
+    digits <- round(digits)
+
+    stopifnot(is.spam(x), is.numeric(digits), length(digits)==1,
+              !rowpointer || (rowpointer && (is.spam(x) || is(x, "spam.chol.NgPeyton"))), # rowpointes are only available for spam
+              is.character(zerosymbol), nchar(zerosymbol) == 1)
+
+    if (length(x@entries) == 0 || ((length(x@entries) == 1) && (x@entries[1] == 0))) {
+        cat("zero-matrix. class: spam (", .format.spam(x)$name, "). dim: ",  x@dimension[1],
+              "x", x@dimension[2], ".", sep = "", fill = TRUE)
+
+        return(invisible(NULL))
+    }
+
+    tmp <- round(x@entries, digits = digits)
+    x@entries <- ifelse(tmp == 0 & x@entries < 0, 0, tmp) # because sprintf() prints "-0"
+
+    if(digits == 0 || any(grep("\\.", format(x@entries, scientific = FALSE)))) {
+        digits <- min(digits, max(nchar(gsub("(.*)(\\.)|([0]*$)","", x@entries))))
+    } else {
+        digits <- 0 }
+
+    width <- max(nchar(as.character(round(abs(x@entries))))) #left side of comma
+    if(any(round(x@entries,digits = digits) < 0)) { # minus sign
+        width <- width+1 }
+    if(digits > 0) {
+        width <- width+1+digits }
+    if(width < 3 && ncol(x) >= 10) { #minus sign or two digits for col number
+        width <- width+1 }
+
+    if(nrow(x) > getOption("spam.printsize") || ncol(x)*(width+1) > getOption("width")[1]) {
+        cat("class: spam (", .format.spam(x)$name, "). dim: ", x@dimension[1], "x",
+            x@dimension[2], ". density: ", signif(100*length(x@entries)/prod(x@dimension),3),
+            "%.\nrow-wise nonzero elements:",   sep = "", fill = TRUE)
+        if(length(x@entries)>10)
+            cat(round(x@entries[1:10], digits), "...\n")
+        else
+            cat(round(x@entries, digits), "\n")
+
+        return(invisible(NULL))
+    }
+
+    headerstring <- paste("Matrix of dimension ",x@dimension[1],"x",
+                          x@dimension[2], " with (row-wise) nonzero elements", sep ="")
+    if (rowpointer)
+      headerstring <- paste(headerstring, ",\n number left of the vertical bars denote the respective rowpointer:\n", sep = "")
+    else
+      headerstring <- paste(headerstring, ":\n", sep = "")
+
+    cat(headerstring)
+    cat("    ", formatC(1:ncol(x), width = width), "\n", sep = " ")
+    cat("    ", rep("-", ncol(x)*(width+1)), "\n" , sep = "")
+    zerostring <- paste(c(rep(" ",width-1), zerosymbol), collapse="")
+    rowtmp <- rep(zerostring, x@dimension[2])
+    for(i in 1:x@dimension[1]){
+        if(rowpointer)
+            cat(formatC(x@rowpointers[i], width = 3), "| ", sep = "")
+        else
+            cat(formatC(i, width = 3), "| ", sep = "")
+        if(x@rowpointers[i]==x@rowpointers[i+1]){
+            cat(rowtmp, "\n")
+            next
+        }
+        row <- rowtmp
+        coli <- x@colindices[x@rowpointers[i]:(x@rowpointers[i+1]-1)]
+        row[coli] <- sprintf(paste0("%", width, ".", digits,"f", collapse=""),
+                             x@entries[x@rowpointers[i]:(x@rowpointers[i+1]-1)],
+                             width = digits+2)
+        cat(row, "\n")
+    }
+    cat(paste0("class: spam (", .format.spam(x)$name, ")\n"))
+}
+
+
+print_nnzpos <- function(x, ...){
+  stopifnot(is.spam(x) || is(x, "spam.chol.NgPeyton"))
+  if(prod(x@dimension) < getOption("spam.printsize"))
+    stop("dim too large for nnzpos=TRUE")
+  x@entries <- seq_along(x@entries)
+  spam::print.spam(x, digits = 0, minimal = FALSE, ...)
+}
+
+
+print.spam <- function(x, ...) {
   SS <- .format.spam(x, validate = FALSE)
   if (prod(x@dimension) < getOption("spam.printsize")) {
-    print(as.matrix.spam(x),...)
+    print(as.matrix.spam(x), ...)
   } else {
     if ( (length(x@entries)==1) && (x@entries[1]==0)) {
       cat("Zero matrix of dimension ",x@dimension[1],"x",
                 x@dimension[2],".\n",sep="", fill=TRUE)
-    }else {
-      cat("Matrix of dimension ",x@dimension[1],"x",
-          x@dimension[2], " with (row-wise) nonzero elements:\n",sep="", fill=TRUE)
-      print(x@entries,...)
+    } else {
+      .print.spam(x, ...)
     }
   }
 
@@ -192,6 +311,7 @@ setMethod("c","spam", function(x,...){
 ########################################################################
 # diag and derivatives
 "diag.spam" <- function(x=1, nrow, ncol)  {
+  ## structure of function according an older version of base:::diag  
   if (is.spam(x)) return( diag.of.spam( x, nrow, ncol))
 
   if (is.array(x) && length(dim(x)) != 1)
@@ -212,13 +332,15 @@ setMethod("c","spam", function(x,...){
 
   m <- min(n, p)
 
-  newx <- new("spam")
-  slot(newx,"entries",check=FALSE) <- vector("double", m)
-  newx@entries[1:m] <- as.double(x)
-  slot(newx,"colindices",check=FALSE) <- 1:m
-  slot(newx,"rowpointers",check=FALSE) <- as.integer(c(1:m,rep(m+1,n+1-m)))
-  slot(newx,"dimension",check=FALSE) <- c(n,p)
-  return(newx)
+  ###CheckMe: are the following two lines efficient?
+  entries <- vector("double", m)
+  entries[1:m] <- x 
+  return(.newSpam(
+      entries=entries,
+        colindices=1:m,
+        rowpointers=c(1:m,rep(m+1,n+1-m)),
+        dimension=c(n,p)
+    ))
 }
 
 spam_diag <- function(x=1, nrow, ncol)  diag.spam(x=x, nrow, ncol)
